/***********************************************************************************************/
/************************************** filter.c ***********************************************/
/***********************************************************************************************/
/*

	written:		1.19.95	by Paul Millman
	
	modified:	8.26.96	by Witaya Wannasuphoprasit
					4.28.00	by Songho M. Kim
					3.22.03  by Eric Faulring
*/

#define __FILTER
#include	"filter.h"

double Filter (FILTER *filt, double xnew)
{
	int			i;
	double		ynew;

	filt->x[filt->index] = xnew;
	filt->y[filt->index] = 0;

	for (i=0; i<=filt->order; i++)
		filt->y[filt->index] += filt->n[i] * filt->x[ (filt->index+i+1)%(filt->order+1) ]
									 - filt->d[i] * filt->y[ (filt->index+i+1)%(filt->order+1) ];

	ynew	= filt->y[filt->index];
	filt->index = (++filt->index) % (filt->order+1);

	return(ynew);
}


void Filter_Vector (FILTER *filt, double vector[], int size)
{
	int			i,j;
	double		xnew, ynew;

	for (j=0; j<size; j++) {

	xnew = vector[j];

	filt->x[filt->index] = xnew;
	filt->y[filt->index] = 0;

	for (i=0; i<=filt->order; i++)
		filt->y[filt->index] += filt->n[i] * filt->x[ (filt->index+i+1)%(filt->order+1) ]
									 - filt->d[i] * filt->y[ (filt->index+i+1)%(filt->order+1) ];

	ynew	= filt->y[filt->index];
	filt->index = (++filt->index) % (filt->order+1);

	vector[j] = ynew;
	}

}


void InitDifferentiator (FILTER *filt, double sample_time)
{
	//	Initializes coefficients for differentiator plus first or
	// second order lowpass filter.

	// You must initialize the following arguments in main:
	//	diff.order, diff.cutoffHz, sample_time

	double	woT;
	double	temp;
	int		i;

	filt->index = 0;
	for (i=0; i<=4; i++) {
		filt->x[i] = 0;
		filt->y[i] = 0;
	}

	if (filt->order == 1) {
		woT = sample_time*filt->cutoffHz*2.0*PI;
		temp = 1.0/(2+woT);

		filt->d[1] = 0.0;
		filt->d[0] = -(2-woT)*temp;

		filt->n[1] = 2.0*filt->cutoffHz*PI*2.0*temp;
		filt->n[0] = -(filt->n[1]);
	}
	else if (filt->order == 2) {
		woT = sample_time*filt->cutoffHz*2.0*PI;
		temp = 1.0 / (4.0*(1.0+woT) + woT*woT);

		filt->d[2] = 0.0;
		filt->d[1] = -(8.0 - 2.0*woT*woT) * temp;
		filt->d[0] = ( 4.0*(1.0-woT) + woT*woT) * temp;

		filt->n[2] = 2.0*filt->cutoffHz*PI*2.0*woT * temp;
		filt->n[1] = 0.0;
		filt->n[0] = -(filt->n[2]);
	}
	else {
		exit (1);
	}
}


void InitLowpass (FILTER *filt, double sample_time, int order, double cutoffHz, double zeta)
{
	//	Initializes coefficients for lowpass filter with 2 complex poles
	// (or real poles if zeta=1).
	// Zeta is the damping ratio, filtcutHz is the cutoff frequency.

	// You must initialize the following arguments in main:
	//	lowpass.order, lowpass.cutoffHz, lowpass.zeta, sample_time

	double	woT;
	double	uu, uu2;
	int		i;
	
	filt->order = order;
	filt->cutoffHz = cutoffHz;
	filt->zeta = zeta;

	filt->index = 0;
	for (i=0; i<=4; i++) {
		filt->x[i] = 0;
		filt->y[i] = 0;
	}


	if (filt->order == 2) {
		woT = sample_time*filt->cutoffHz*2.0*PI;
		uu	 = 2.0/woT;
		uu2 = uu*uu;

		filt->d[2] = (uu2 + 2.0*filt->zeta*uu + 1.0);
		filt->d[1] = -(2.0*uu2-2.0)/filt->d[2];
		filt->d[0] = ( uu2 - 2.0*filt-> zeta*uu + 1.0 )/filt->d[2];

		filt->n[2] = 1.0/filt->d[2];
		filt->n[1] = 2.0/filt->d[2];
		filt->n[0] = 1.0/filt->d[2];

		filt->d[2] = 0.0;
	}
	else {
		exit (1);
	}
}




