/***********************************************************************************************/
/************************************** filter.h ***********************************************/
/***********************************************************************************************/
/*

	written:		1.19.95	by Paul Millman
	
	modified:	8.26.96	by Witaya Wannasuphoprasit
					4.28.00	by Songho M. Kim
					3.22.03  by Eric Faulring
*/


#ifndef FILTER_H
#define FILTER_H

/***********************************************************************************************/
/************************************ Standard Headers *****************************************/
/***********************************************************************************************/
#ifdef __FILTER
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#define	PI	3.141592654
#endif

#define	MAX_FILT_SIZE	5		// max size of filter coefficient arrays
#define	FIRST_ORDER		1
#define	SECOND_ORDER	2
#define	FOURTH_ORDER	4

typedef struct FILTER {
	int		order;			// FIRST_ORDER, SECOND_ORDER, OR FOURTH_ORDER
	double	cutoffHz;			// cutoff frequency in Hz
	double	zeta;				// damping factor (between 0 and 1)
	double	d[MAX_FILT_SIZE];		// coefficients of denominator
	double	n[MAX_FILT_SIZE];		// coefficients of numerator
	double	x[MAX_FILT_SIZE];		// old input values
	double	y[MAX_FILT_SIZE];		// old output values
	int		index;			// index of current value in input and output arrays
} FILTER;


/* function prototype */
double	Filter (FILTER *filt, double xnew);
void		Filter_Vector (FILTER *filt, double vector[], int size);
void		InitDifferentiator (FILTER *filt, double sample_time);
void		InitLowpass (FILTER *filt, double sample_time, int order, double cutoffHz, double zeta);

#endif
