/***********************************************************************************************/
/******************************** gui.c ********************************************************/
/***********************************************************************************************/

#define __GUI
#include "gui.h"

/***********************************************************************************************/
/************************************ StaticInputWindow() **************************************/
/***********************************************************************************************/
void StaticInputWindow(){

	int row;
	int col;

	/* border */
	wattron(InputWindow,COLOR_PAIR(WB)|A_BOLD);
	wborder(InputWindow,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);			

   row = 1;
	wattron(InputWindow,COLOR_PAIR(YB)|A_BOLD);


   /* axis 01 information */   
   mvwprintw(InputWindow,row++,INPUT_col0,  "AXIS 01 cur pos (cnts):");
   mvwprintw(InputWindow,row++,INPUT_col0,  "AXIS 01 pre pos (rads):");
   mvwprintw(InputWindow,row++,INPUT_col0,  "AXIS 01 cur pos (rads):");
   mvwprintw(InputWindow,row++,INPUT_col0,  "AXIS 01 cur vel (rd/s):");
   mvwprintw(InputWindow,row++,INPUT_col0,  "AXIS 01 fil vel (rd/s):");

   row ++;
   /* chan 01 information */   
   mvwprintw(InputWindow,row++,INPUT_col0,  "CHAN 01 cur pos (volts):");
   mvwprintw(InputWindow,row++,INPUT_col0,  "CHAN 01 pre pos (units):");
   mvwprintw(InputWindow,row++,INPUT_col0,  "CHAN 01 cur pos (units):");
   mvwprintw(InputWindow,row++,INPUT_col0,  "CHAN 01 cur vel (units/s):");
   mvwprintw(InputWindow,row++,INPUT_col0,  "CHAN 01 fil vel (units/s):");

	wrefresh(InputWindow);
}
/***********************************************************************************************/
/************************************ DisplayInputWindow() *************************************/
/***********************************************************************************************/
void DisplayInputWindow(){

	int row;
	int col;

   row = 1;
	wattron(InputWindow,COLOR_PAIR(CB)|A_BOLD); 
   
   /* axis 01 information */ 
	sprintf(tmpstr,"% 9d",raw_STG_ENC_data.cnts[AXIS_01_ENC].Long);
	mvwprintw(InputWindow,row++,INPUT_col1,tmpstr);
	sprintf(tmpstr,"% 9.4f",axis_01_p_pos);
	mvwprintw(InputWindow,row++,INPUT_col1,tmpstr);
	sprintf(tmpstr,"% 9.4f",axis_01_c_pos);
	mvwprintw(InputWindow,row++,INPUT_col1,tmpstr);
	sprintf(tmpstr,"% 9.4f",axis_01_c_vel);
	mvwprintw(InputWindow,row++,INPUT_col1,tmpstr);
	sprintf(tmpstr,"% 9.4f",axis_01_f_vel);
	mvwprintw(InputWindow,row++,INPUT_col1,tmpstr);

   row ++;
   /* chan 01 information */ 
	sprintf(tmpstr,"% 9.4f",raw_STG_ADC_data[CHAN_01_ADC]);
	mvwprintw(InputWindow,row++,INPUT_col1,tmpstr);
	sprintf(tmpstr,"% 9.4f",chan_01_p_pos);
	mvwprintw(InputWindow,row++,INPUT_col1,tmpstr);
	sprintf(tmpstr,"% 9.4f",chan_01_c_pos);
	mvwprintw(InputWindow,row++,INPUT_col1,tmpstr);
	sprintf(tmpstr,"% 9.4f",chan_01_c_vel);
	mvwprintw(InputWindow,row++,INPUT_col1,tmpstr);
	sprintf(tmpstr,"% 9.4f",chan_01_f_vel);
	mvwprintw(InputWindow,row++,INPUT_col1,tmpstr);

	wrefresh(InputWindow);
}
/***********************************************************************************************/
/************************************ StaticOutputWindow() ********************************/
/***********************************************************************************************/
void StaticOutputWindow(){

	int row = 1;
	int col;
	int i;
			
	/* border */
	wattron(OutputWindow,COLOR_PAIR(WB)|A_BOLD);
	wborder(OutputWindow,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);

   /* AXIS 01 dac output labels */
   row = 5;
	wattron(OutputWindow,COLOR_PAIR(YB)|A_BOLD);
   mvwprintw(OutputWindow,row++,OUTPUT_col0,  "AXIS 01 cur cmd (volts):");

	wrefresh(OutputWindow);

}	
/***********************************************************************************************/
/************************************ DisplayOutputWindow() *******************************/
/***********************************************************************************************/
void DisplayOutputWindow(){

	int row;
	int col;

   /* labels for adjustable parameters - they change color so they can't be defined statically */
	row = 1;
	col = OUTPUT_col0;
	wattroff(OutputWindow,A_BOLD);
	wattron(OutputWindow,COLOR_PAIR(WB));

	if (FIELD_TO_ADJUST == ADJ_P_GAIN) wattron(OutputWindow,A_BLINK);
	else wattroff(OutputWindow,A_BLINK);
	sprintf(tmpstr,"AXIS 01 P gain:");
	mvwprintw(OutputWindow,row++,col,tmpstr);

	if (FIELD_TO_ADJUST == ADJ_I_GAIN) wattron(OutputWindow,A_BLINK);
	else wattroff(OutputWindow,A_BLINK);
	sprintf(tmpstr,"AXIS 01 I gain:");
	mvwprintw(OutputWindow,row++,col,tmpstr);

	if (FIELD_TO_ADJUST == ADJ_D_GAIN) wattron(OutputWindow,A_BLINK);
	else wattroff(OutputWindow,A_BLINK);
	sprintf(tmpstr,"AXIS 01 D gain:");
	mvwprintw(OutputWindow,row++,col,tmpstr);

   /* the adjustable parameter values */
   wattroff(OutputWindow,A_BLINK);
	wattron(OutputWindow,COLOR_PAIR(YB)|A_BOLD);
   row = 1;
	sprintf(tmpstr,"% 9.4f",axis_01_p_gain);
	mvwprintw(OutputWindow,row++,OUTPUT_col1,tmpstr);
	sprintf(tmpstr,"% 9.4f",axis_01_i_gain);
	mvwprintw(OutputWindow,row++,OUTPUT_col1,tmpstr);
	sprintf(tmpstr,"% 9.4f",axis_01_d_gain);
	mvwprintw(OutputWindow,row++,OUTPUT_col1,tmpstr);

   /* AXIS 01 dac output */
   row = 5;
	wattron(OutputWindow,COLOR_PAIR(CB)|A_BOLD);
	sprintf(tmpstr,"% 9.4f",axis_01_cmd_voltage);
	mvwprintw(OutputWindow,row++,OUTPUT_col1,tmpstr);

	wrefresh(OutputWindow);
}

/***********************************************************************************************/
/************************************* UpdateScreen() ******************************************/
/***********************************************************************************************/
void UpdateScreen(){

   time_t time_of_day;
      
		/* update the message window */
		DisplayErrorMsg();
		/* populate the cobot window */
      DisplayInputWindow();
		/* populate the environment window */
      DisplayOutputWindow();
		
      /* update the clock */
      time_of_day = time( NULL );
      strftime( tmpstr, 60, "%r",localtime( &time_of_day ) );
		wattron(DateTimeWindow,COLOR_PAIR(CB)|A_BOLD);
      mvwprintw(DateTimeWindow,1,25,tmpstr);
      wrefresh(DateTimeWindow);		

		/* update the heartbeat */
		sprintf( tmpstr, "% d", beat);
		wattron(HeartBeatWindow,COLOR_PAIR(CB)|A_BOLD);
		mvwprintw(HeartBeatWindow,1,18,tmpstr);
      wrefresh(HeartBeatWindow);	

		/* update the filename */
		if(recording == TRUE) {
			wattron(FileNameWindow,COLOR_PAIR(CB)|A_BOLD|A_BLINK);
			mvwprintw(FileNameWindow,1,22,filestr);
			wattroff(FileNameWindow,A_BOLD|A_BLINK);
			wrefresh(FileNameWindow);
		}
		else {
			wattron(FileNameWindow,COLOR_PAIR(CB)|A_BOLD);
			mvwprintw(FileNameWindow,1,22,filestr);
			wattroff(FileNameWindow,A_BOLD);
			wrefresh(FileNameWindow);
		}

		/* update the main mode */
		switch (main_status){
			case (WAITING):
				sprintf(tmpstr,"          Waiting ");
			break;
			case (FAULT):
				sprintf(tmpstr,"            Fault ");
			break;
			case (TEST_MODE):
				sprintf(tmpstr,"        Test Mode ");
			break;
			default:
				sprintf(tmpstr,"     Mode Unknown ");		
			break;
		}
		wattron(MainStatusWindow,COLOR_PAIR(CB)|A_BOLD);
		mvwprintw(MainStatusWindow,1,27,tmpstr);
      wrefresh(MainStatusWindow);

		/* update the filesize */
		wattron(FileNameWindow,COLOR_PAIR(CB)|A_BOLD);
		mvwprintw(FileNameWindow,1,64,filesizestr);
		wrefresh(FileNameWindow);
		/* update the interrupt frequency - this never changes */
		sprintf( tmpstr, "% 5.3f", DT_INV);
		wattron(InterruptWindow,COLOR_PAIR(CB)|A_BOLD);
		mvwprintw(InterruptWindow,1,27,tmpstr);
		wrefresh(InterruptWindow);
		/* update the timestep - this never changes */
		sprintf( tmpstr,"% 3.4f",DT*1e06);
		wattron(TimeStepWindow,COLOR_PAIR(CB)|A_BOLD);
		mvwprintw(TimeStepWindow,1,27,tmpstr);
		wrefresh(TimeStepWindow);
		/* update the cpu cycles - this never changes */
		sprintf( tmpstr, "% 1.6f", (float)(cps/(1e9)));
		wattron(CPUWindow,COLOR_PAIR(CB)|A_BOLD);
		mvwprintw(CPUWindow,1,27,tmpstr);
		wrefresh(CPUWindow);	
	
}
/***********************************************************************************************/
/************************************ InitializeDisplay() **************************************/
/**************** this is called from main.c to set up the size of window segments *************/
/***********************************************************************************************/
void InitializeDisplay()
{
	int i;

	/* first line */
	TitleWindow = subwin(MainWindow, 3, TITLE_WIDTH, 0,0);
   /* second line */
	DateTimeWindow = subwin(MainWindow, 3, DATETIME_WIDTH, 3,0);
	InterruptWindow = subwin(MainWindow, 3, INTERRUPT_WIDTH, 3,DATETIME_WIDTH);
   /* third line */
	TimeStepWindow = subwin(MainWindow, 3, TIMESTEP_WIDTH, 6,0);
	CPUWindow = subwin(MainWindow, 3, CPU_WIDTH, 6,TIMESTEP_WIDTH);
	/* large data windows */
	InputWindow = subwin(MainWindow, INPUT_WIN_HEIGHT , INPUT_WIN_WIDTH, 9,0);
	OutputWindow = subwin(MainWindow, OUTPUT_WIN_HEIGHT , OUTPUT_WIN_WIDTH, 9,INPUT_WIN_WIDTH);
   /* message window */
	ErrorWindow = subwin(MainWindow, ERR_WIN_HEIGHT, ERR_WIN_WIDTH, INPUT_WIN_HEIGHT+9,0);
   /* second to last line */
	MainStatusWindow = subwin(MainWindow,3,MAINSTATUS_WIDTH,INPUT_WIN_HEIGHT+ERR_WIN_HEIGHT+9,0);
	HeartBeatWindow = subwin(MainWindow, 3, HEARTBEAT_WIDTH, INPUT_WIN_HEIGHT+ERR_WIN_HEIGHT+9,MAINSTATUS_WIDTH);
   /* last line */
	FileNameWindow = subwin(MainWindow, 3, FILENAME_WIDTH, INPUT_WIN_HEIGHT+ERR_WIN_HEIGHT+12,0);


}
/***********************************************************************************************/
/*************************************** DrawStaticInfo() **************************************/
/************** this is called from main.c or whenever you hit "d" on the keyboard *************/
/***********************************************************************************************/
void DrawStaticInfo()
{

	StaticTitleFootnoteWindows();
	StaticErrorWindow();
   StaticInputWindow();
	StaticOutputWindow();

}

/***********************************************************************************************/
/********************************** StaticEnvironmentWindow() *****************************************/
/***********************************************************************************************/
void StaticTitleFootnoteWindows(){

	wattron(TitleWindow,COLOR_PAIR(WB)|A_BOLD);
   wborder(TitleWindow,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
   wattron(TitleWindow,COLOR_PAIR(YB));
   mvwprintw(TitleWindow,1,3,  "Hand Controller Program, v1.0, June 2003, Eric Faulring");
	wattroff(TitleWindow,A_BOLD);

   wattron(DateTimeWindow,COLOR_PAIR(WB)|A_BOLD);
   wborder(DateTimeWindow,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
	wattron(DateTimeWindow,COLOR_PAIR(YB));
   mvwprintw(DateTimeWindow,1,1,"Current Time:");
	wattroff(DateTimeWindow,A_BOLD);

   wattron(InterruptWindow,COLOR_PAIR(WB)|A_BOLD);
   wborder(InterruptWindow,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
	wattron(InterruptWindow,COLOR_PAIR(YB));
   mvwprintw(InterruptWindow,1,1,"Interrupt Frequency (Hz):");
	wattroff(InterruptWindow,A_BOLD);

   wattron(TimeStepWindow,COLOR_PAIR(WB)|A_BOLD);
   wborder(TimeStepWindow,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
	wattron(TimeStepWindow,COLOR_PAIR(YB));
	mvwprintw(TimeStepWindow,1,1,"TimeStep (us):");
	wattroff(TimeStepWindow,A_BOLD);

   wattron(CPUWindow,COLOR_PAIR(WB)|A_BOLD);
   wborder(CPUWindow,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
	wattron(CPUWindow,COLOR_PAIR(YB));
	mvwprintw(CPUWindow,1,1,"CPU Cycles (cpns):");
	wattroff(CPUWindow,A_BOLD);

   wattron(MainStatusWindow,COLOR_PAIR(WB)|A_BOLD);
   wborder(MainStatusWindow,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
	wattron(MainStatusWindow,COLOR_PAIR(YB));
	mvwprintw(MainStatusWindow,1,1,"Program Main Status:");
	wattroff(MainStatusWindow,A_BOLD);

   wattron(FileNameWindow,COLOR_PAIR(WB)|A_BOLD);
   wborder(FileNameWindow,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
	wattron(FileNameWindow,COLOR_PAIR(YB));
	mvwprintw(FileNameWindow,1,1,"Current Record Name:                              Filesize:");
	wattroff(FileNameWindow,A_BOLD);

   wattron(HeartBeatWindow,COLOR_PAIR(WB)|A_BOLD);
   wborder(HeartBeatWindow,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
	wattron(HeartBeatWindow,COLOR_PAIR(YB));
	mvwprintw(HeartBeatWindow,1,1,"HeartBeat (cnts):");
	wattroff(HeartBeatWindow,A_BOLD);
}

/***********************************************************************************************/
/************************************* StaticErrorWindow() *************************************/
/***********************************************************************************************/
void StaticErrorWindow(){
   wattron(ErrorWindow,COLOR_PAIR(WB)|A_BOLD);
   wborder(ErrorWindow,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
   wattroff(ErrorWindow,A_BOLD);
}

/***********************************************************************************************/
/********************************** deallocate windows *****************************************/
/***********************************************************************************************/
void Free_Windows(){

	int i;

	delwin(TitleWindow);  
	delwin(DateTimeWindow);
	delwin(InterruptWindow);
	delwin(TimeStepWindow);
	delwin(CPUWindow);
	delwin(HeartBeatWindow);
   delwin(ErrorWindow);
	delwin(InputWindow);
	delwin(OutputWindow);
	delwin(MainStatusWindow);
	delwin(MainWindow);

}

/***********************************************************************************************/
/************************************ PrintErrorMsg() ******************************************/
/******************************* don't mess with this code *************************************/
/***********************************************************************************************/
void PrintErrorMsg(char* msg)
{
	char buffer[250];
	char timestr[50];
	char heartbeatstr[50];
	int i;
   time_t time_of_day;

		/* record a new message */
		if (++msg_idx == SIZE_MSG_BUFFER) msg_idx = 0;
		sprintf(buffer,"%4d   ",++msg_cnt);
		time_of_day = time( NULL );
		strftime( timestr, 40, "%j-%H:%M:%S ",localtime( &time_of_day ) );
		strcat(buffer,timestr);
		sprintf(heartbeatstr,"%10d   ",beat);
		strcat(buffer,heartbeatstr);
		strcat(buffer,msg);
		sprintf(old_msgs[msg_idx].msg,buffer);
	      
}
/***********************************************************************************************/
/************************************ DisplayErrorMsg() ****************************************/
/******************************* don't mess with this code *************************************/
/***********************************************************************************************/
void DisplayErrorMsg()
{
	int i;
	static int last_msg_displayed = -1;

	/* if new messages blow away any scrolling and print new messages */
	if (last_msg_displayed != msg_cnt){
		/* clear the error window without removing the border */
      wmove(ErrorWindow,1,1);
		for (i=0;i<(ERR_WIN_HEIGHT-3);i++) mvwhline(ErrorWindow,i+1,1,(chtype)' ',ERR_WIN_WIDTH-2);
     
		/* check for color */
      if(has_colors() == TRUE) wattron(ErrorWindow,COLOR_PAIR(YB)|A_BOLD);

		for (i=0;i<(ERR_WIN_HEIGHT-3)-1;i++){
			if (msg_cnt > i) {
				if (msg_idx-1-i < 0) mvwprintw(ErrorWindow,(ERR_WIN_HEIGHT-3)-1-i,1,old_msgs[SIZE_MSG_BUFFER+(msg_idx-1-i)].msg);
				else mvwprintw(ErrorWindow,(ERR_WIN_HEIGHT-3)-1-i,1,old_msgs[msg_idx-1-i].msg);
			}
		}
      mvwprintw(ErrorWindow,(ERR_WIN_HEIGHT-3),1,old_msgs[msg_idx].msg);

		/* reinitialize the scrolling index since we jumped to the most recent message */
      visible_msg_idx = 0;

		/* move the cursor */
      wmove(ErrorWindow,(ERR_WIN_HEIGHT-3)+1,1);

		wrefresh(ErrorWindow);
		last_msg_displayed = msg_cnt;
	}
      
}
/***********************************************************************************************/
/************************************ ScrollErrorMsg() *****************************************/
/******************************* don't mess with this code *************************************/
/***********************************************************************************************/
void ScrollErrorMsg(int updown)
{
	int print_msg_idx = 0;
	int i;

		/* update the index of where in the buffer of messages you are */
		visible_msg_idx += updown;
		if (visible_msg_idx > 0) visible_msg_idx = 0;
		if (msg_cnt < (ERR_WIN_HEIGHT-3)) visible_msg_idx = 0;
		if ((visible_msg_idx < 0) && (msg_cnt <= -visible_msg_idx + (ERR_WIN_HEIGHT-3)-1)) visible_msg_idx = -msg_cnt+(ERR_WIN_HEIGHT-3)-1;
		if (visible_msg_idx < (-SIZE_MSG_BUFFER + (ERR_WIN_HEIGHT-3))) visible_msg_idx = -SIZE_MSG_BUFFER + (ERR_WIN_HEIGHT-3);

		/* clear the error window without removing the border */
      wmove(ErrorWindow,1,1);
		for (i=0;i<(ERR_WIN_HEIGHT-3);i++) mvwhline(ErrorWindow,i+1,1,(chtype)' ',ERR_WIN_WIDTH-2);
      
		print_msg_idx = msg_idx + visible_msg_idx;
		if (print_msg_idx < 0) print_msg_idx = SIZE_MSG_BUFFER + print_msg_idx;

		/* check for color */
      if(has_colors() == TRUE) wattron(ErrorWindow,COLOR_PAIR(YB)|A_BOLD);


		for (i=0;i<(ERR_WIN_HEIGHT-3)-1;i++){
			if (msg_cnt > i) {
				if (print_msg_idx-1-i < 0) mvwprintw(ErrorWindow,(ERR_WIN_HEIGHT-3)-1-i,1,old_msgs[SIZE_MSG_BUFFER+(print_msg_idx-1-i)].msg);
				else mvwprintw(ErrorWindow,(ERR_WIN_HEIGHT-3)-1-i,1,old_msgs[print_msg_idx-1-i].msg);
			}
		}
      mvwprintw(ErrorWindow,(ERR_WIN_HEIGHT-3),1,old_msgs[print_msg_idx].msg);
      
		/* move the cursor */
      wmove(ErrorWindow,(ERR_WIN_HEIGHT-3)+1,1);

		wrefresh(ErrorWindow);
  
}